/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)ufs_tables.c	7.3 (Berkeley) 4/24/89
 */

#include <afsconfig.h>
#include <afs/param.h>

RCSID
    ("$Header: /cvs/openafs/src/vfsck/ufs_tables.c,v 1.4 2003/07/15 23:17:27 shadow Exp $");

#ifdef KERNEL
#include "param.h"
#else
#include <sys/param.h>
#ifdef 	AFS_HPUX_ENV
#include <sys/fs.h>
#endif
#endif

/*
 * Bit patterns for identifying fragments in the block map
 * used as ((map & around) == inside)
 */
int around[9] = {
    0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, 0xff, 0x1ff, 0x3ff
};
int inside[9] = {
    0x0, 0x2, 0x6, 0xe, 0x1e, 0x3e, 0x7e, 0xfe, 0x1fe
};

/*
 * Given a block map bit pattern, the frag tables tell whether a
 * particular size fragment is available. 
 *
 * used as:
 * if ((1 << (size - 1)) & fragtbl[fs->fs_frag][map] {
 *	at least one fragment of the indicated size is available
 * }
 *
 * These tables are used by the scanc instruction on the VAX to
 * quickly find an appropriate fragment.
 */
u_char fragtbl124[256] = {
    0x00, 0x16, 0x16, 0x2a, 0x16, 0x16, 0x26, 0x4e,
    0x16, 0x16, 0x16, 0x3e, 0x2a, 0x3e, 0x4e, 0x8a,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x2a, 0x3e, 0x3e, 0x2a, 0x3e, 0x3e, 0x2e, 0x6e,
    0x3e, 0x3e, 0x3e, 0x3e, 0x2a, 0x3e, 0x6e, 0xaa,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x26, 0x36, 0x36, 0x2e, 0x36, 0x36, 0x26, 0x6e,
    0x36, 0x36, 0x36, 0x3e, 0x2e, 0x3e, 0x6e, 0xae,
    0x4e, 0x5e, 0x5e, 0x6e, 0x5e, 0x5e, 0x6e, 0x4e,
    0x5e, 0x5e, 0x5e, 0x7e, 0x6e, 0x7e, 0x4e, 0xce,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x16, 0x16, 0x16, 0x3e, 0x16, 0x16, 0x36, 0x5e,
    0x16, 0x16, 0x16, 0x3e, 0x3e, 0x3e, 0x5e, 0x9e,
    0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x7e,
    0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x7e, 0xbe,
    0x2a, 0x3e, 0x3e, 0x2a, 0x3e, 0x3e, 0x2e, 0x6e,
    0x3e, 0x3e, 0x3e, 0x3e, 0x2a, 0x3e, 0x6e, 0xaa,
    0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x7e,
    0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x3e, 0x7e, 0xbe,
    0x4e, 0x5e, 0x5e, 0x6e, 0x5e, 0x5e, 0x6e, 0x4e,
    0x5e, 0x5e, 0x5e, 0x7e, 0x6e, 0x7e, 0x4e, 0xce,
    0x8a, 0x9e, 0x9e, 0xaa, 0x9e, 0x9e, 0xae, 0xce,
    0x9e, 0x9e, 0x9e, 0xbe, 0xaa, 0xbe, 0xce, 0x8a,
};

u_char fragtbl8[256] = {
    0x00, 0x01, 0x01, 0x02, 0x01, 0x01, 0x02, 0x04,
    0x01, 0x01, 0x01, 0x03, 0x02, 0x03, 0x04, 0x08,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x02, 0x03, 0x03, 0x02, 0x04, 0x05, 0x08, 0x10,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x01, 0x01, 0x01, 0x03, 0x03, 0x03, 0x05, 0x09,
    0x02, 0x03, 0x03, 0x02, 0x03, 0x03, 0x02, 0x06,
    0x04, 0x05, 0x05, 0x06, 0x08, 0x09, 0x10, 0x20,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x01, 0x01, 0x01, 0x03, 0x03, 0x03, 0x05, 0x09,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x03, 0x03, 0x03, 0x03, 0x05, 0x05, 0x09, 0x11,
    0x02, 0x03, 0x03, 0x02, 0x03, 0x03, 0x02, 0x06,
    0x03, 0x03, 0x03, 0x03, 0x02, 0x03, 0x06, 0x0a,
    0x04, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x04,
    0x08, 0x09, 0x09, 0x0a, 0x10, 0x11, 0x20, 0x40,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x01, 0x01, 0x01, 0x03, 0x03, 0x03, 0x05, 0x09,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x03, 0x03, 0x03, 0x03, 0x05, 0x05, 0x09, 0x11,
    0x01, 0x01, 0x01, 0x03, 0x01, 0x01, 0x03, 0x05,
    0x01, 0x01, 0x01, 0x03, 0x03, 0x03, 0x05, 0x09,
    0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x07,
    0x05, 0x05, 0x05, 0x07, 0x09, 0x09, 0x11, 0x21,
    0x02, 0x03, 0x03, 0x02, 0x03, 0x03, 0x02, 0x06,
    0x03, 0x03, 0x03, 0x03, 0x02, 0x03, 0x06, 0x0a,
    0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x07,
    0x02, 0x03, 0x03, 0x02, 0x06, 0x07, 0x0a, 0x12,
    0x04, 0x05, 0x05, 0x06, 0x05, 0x05, 0x06, 0x04,
    0x05, 0x05, 0x05, 0x07, 0x06, 0x07, 0x04, 0x0c,
    0x08, 0x09, 0x09, 0x0a, 0x09, 0x09, 0x0a, 0x0c,
    0x10, 0x11, 0x11, 0x12, 0x20, 0x21, 0x40, 0x80,
};

/*
 * The actual fragtbl array.
 */
u_char *fragtbl[MAXFRAG + 1] = {
    0, fragtbl124, fragtbl124, 0, fragtbl124, 0, 0, 0, fragtbl8,
};
